#!/usr/bin/perl

use vars qw(%config);
use CGI qw(:standard);
use Time::Local;
use POSIX;

my $PATHPREFIX = "/usr/local/glotv";
do "$PATHPREFIX/etc/glotv.conf" or die "Failed reading configuration file";

my $GLOTV         = "$PATHPREFIX/glotv";
my $GLOTVSCHEDULE = "$PATHPREFIX/glotvschedule-wrapper";

print header('application/xhtml+xml');
print qq(<?xml version="1.0" encoding="utf-8"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">

<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en-US">
<head>
  <title>GLOTV Recording Schedule</title>
  <style type="text/css">
	table {
		border-collapse: collapse; text-align: center;
		margin-top: 1ex; margin-bottom: 1ex;
	}
	table.listing * th {
		padding-left: 1em; padding-right: 1em;
	}
	table.listing * td {
		border: 2px solid white; padding-left: 1em; padding-right: 1em;
		background-color: #dcdad5;
	}
	table.rec {
		background-color: #dcdad5;
	}
	table.rec * td {
		padding-left: 1em; padding-right: 1em; padding-bottom: 2ex;
	}
  </style>
</head>
<body>
<h1>GLOTV Recording Schedule</h1>
);

my @param_select = multi_param('select');
my $param_start = param('start');
my $param_until = param('until');
my $param_channel = param('channel');
my $param_tuner = param('tuner');
my $param_rec_tag = param('rec_tag');

if (!defined($param_tuner)) {
    $param_tuner = $config{webgui_default_tuner};
}

sub timefmt {
    my $t = shift;
    my ($second,$minute,$hour,$day,$month,$year) = localtime($t);
    return sprintf("%04d-%02d-%02d <b>%02d:%02d</b>",
		   $year+1900, $month+1, $day, $hour, $minute);
}

sub print_channel_options {
    my $current_channel = shift;
    my $cmd = "$GLOTV --command=list_channels";
    $cmd .= " --tuner=\"$param_tuner\"" if $param_tuner;
    my $list = `$cmd`;
    chomp $list;
    foreach my $line (split/^/m, $list) {
	my @row = split(/ /, $line, 3);
	my $opt_selected = ($current_channel eq $row[0] ? " selected='selected'" : "");
	print "<option value='$row[0]'$opt_selected>$row[0]. ".escapeHTML($row[2])."</option>";
    }
}

if (request_method eq 'POST') {
    $param_start =~ tr'"$\\''d;
    $param_until =~ tr'"$\\''d;
    $param_channel =~ tr'"$\\''d;
    $param_rec_tag =~ tr'"$\\''d;
    my $cmd = "$GLOTVSCHEDULE --start=\"$param_start\"";
    $cmd .= " --until=\"$param_until\"" if $param_until;
    $cmd .= " --channel=\"$param_channel\"" if $param_channel;
    $cmd .= " --tuner=\"$param_tuner\"" if $param_tuner;
    $cmd .= " --rec-tag=\"$param_rec_tag\"" if $param_rec_tag;
    if (param('validate')) {
	my $canon = `$cmd --canonize`;
	chomp $canon;
	($param_start,$param_until,$param_channel,$param_tuner,$param_rec_tag) = split(/,/, $canon);
    } elsif (param('cancel')) {
	if (@param_select) {
	    system "$GLOTVSCHEDULE --cancel=" . join(",", grep(m/^\d+-\d+$/, @param_select));
	}
    } elsif (param('submit')) {
	if ($param_start) {
	    system $cmd;
	}
    }
}

my @schedlines = `$GLOTVSCHEDULE --list`;

print '<form method="post" action="">';

if (@schedlines) {
    print "<table class='listing'>\n";
    print "<tr>";
    print "<th>Start time</th>";
    print "<th>Stop time</th>";
    print "<th>Channel</th>";
    print "<th>Tuner</th>";
    print "<th>Tag</th>";
    print '<th><input type="submit" name="cancel" value="Cancel selected recordings"/></th>';
    print "</tr>\n";
    foreach my $line (@schedlines) {
	my ($starttime,$stoptime,$channel,$tuner,$rec_tag) = split(/,/, $line);
	print "<tr>";
	print "<td>", &timefmt($starttime), "</td>";
	print "<td>", &timefmt($stoptime), "</td>";
	print "<td>", escapeHTML($channel), "</td>";
	print "<td>", escapeHTML($tuner), "</td>";
	print "<td>", escapeHTML($rec_tag), "</td>";
	my $ck = (grep($_ eq $starttime, @param_select) ? " checked='true'" : "");
	print "<td><input type='checkbox' name='select' value='$starttime-$tuner'$ck/></td>";
	print "</tr>\n";
    }
    print "</table>\n";
}

print "Schedule new recording:<br/>\n";
print "<table class='rec'>\n";
print "<tr>";
print "<th>Start</th>";
print "<th>Until</th>";
print "<th>Channel</th>";
print "<th>Tuner</th>";
print "<th>Tag</th>";
print "</tr>\n";
print "<tr>\n";
print '<td><input type="text" name="start" size="16" value="'.escapeHTML($param_start).'" /></td>';
print '<td><input type="text" name="until" size="16" value="'.escapeHTML($param_until).'" /></td>';
print '<td><select name="channel">';
&print_channel_options($param_channel);
print '</select></td>';
print '<td><input type="text" name="tuner" size="2" value="'.escapeHTML($param_tuner).'" /></td>';
print '<td><input type="text" name="rec_tag" size="12" value="'.escapeHTML($param_rec_tag).'" /></td>';
print "</tr>\n";
print "</table>\n";

print '<input type="submit" name="validate" value="Validate"/>';
print "\n";
print '<input type="submit" name="submit" value="Submit"/>';
print "</form>\n";

print '<p><a href="tv.pl">Clear all inputs</a></p>';

print qq(
</body>
</html>
);

